# -*- coding: utf-8 -*-
"""
    wakatime.projects.subversion
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Information about the svn project for a given file.

    :copyright: (c) 2013 Alan Hamlett.
    :license: BSD, see LICENSE for more details.
"""

import logging
import os
import platform
from subprocess import Popen, PIPE

from .base import BaseProject
try:
    from collections import OrderedDict
except ImportError:
    from ..packages.ordereddict import OrderedDict


log = logging.getLogger(__name__)


# str is unicode in Python3
try:
    unicode
except NameError:
    unicode = str


class Subversion(BaseProject):

    def process(self):
        return self._find_project_base(self.path)

    def name(self):
        return unicode(self.info['Repository Root'].split('/')[-1])

    def branch(self):
        if self.base:
            unicode(os.path.basename(self.base))
        return None

    def _get_info(self, path):
        info = OrderedDict()
        stdout = None
        try:
            os.environ['LANG'] = 'en_US'
            stdout, stderr = Popen([
                'svn', 'info', os.path.realpath(path)
            ], stdout=PIPE, stderr=PIPE).communicate()
        except OSError:
            pass
        else:
            if stdout:
                interesting = [
                    'Repository Root',
                    'Repository UUID',
                    'URL',
                ]
                for line in stdout.splitlines():
                    if isinstance(line, bytes):
                        line = bytes.decode(line)
                    line = line.split(': ', 1)
                    if line[0] in interesting:
                        info[line[0]] = line[1]
        return info

    def _find_project_base(self, path, found=False):
        if platform.system() == 'Windows':
            return False
        path = os.path.realpath(path)
        if os.path.isfile(path):
            path = os.path.split(path)[0]
        info = self._get_info(path)
        if len(info) > 0:
            found = True
            self.base = path
            self.info = info
        elif found:
            return True
        split_path = os.path.split(path)
        if split_path[1] == '':
            return found
        return self._find_project_base(split_path[0], found)




# -*- coding: utf-8 -*-
"""
    wakatime.queue
    ~~~~~~~~~~~~~~

    Queue for offline time logging.
    http://wakatime.com

    :copyright: (c) 2014 Alan Hamlett.
    :license: BSD, see LICENSE for more details.
"""


import logging
import os
import sqlite3
from time import sleep


log = logging.getLogger(__name__)


class Queue(object):
    DB_FILE = os.path.join(os.path.expanduser('~'), '.wakatime.db')

    def connect(self):
        exists = os.path.exists(self.DB_FILE)
        conn = sqlite3.connect(self.DB_FILE)
        c = conn.cursor()
        c.execute('''CREATE TABLE IF NOT EXISTS action (
            file text,
            time real,
            project text,
            language text,
            lines integer,
            branch text,
            is_write integer,
            plugin text)
        ''')
        return (conn, c)


    def push(self, data, plugin):
        conn, c = self.connect()
        action = {
            'file': data.get('file'),
            'time': data.get('time'),
            'project': data.get('project'),
            'language': data.get('language'),
            'lines': data.get('lines'),
            'branch': data.get('branch'),
            'is_write': 1 if data.get('is_write') else 0,
            'plugin': plugin,
        }
        c.execute('INSERT INTO action VALUES (:file,:time,:project,:language,:lines,:branch,:is_write,:plugin)', action)
        conn.commit()
        conn.close()


    def pop(self):
        tries = 3
        wait = 0.1
        action = None
        conn, c = self.connect()
        loop = True
        while loop and tries > -1:
            try:
                c.execute('BEGIN IMMEDIATE')
                c.execute('SELECT * FROM action LIMIT 1')
                row = c.fetchone()
                if row is not None:
                    c.execute('''DELETE FROM action WHERE
                        file=? AND time=? AND project=? AND language=? AND
                        lines=? AND branch=? AND is_write=?''', row[0:7])
                conn.commit()
                if row is not None:
                    action = {
                        'file': row[0],
                        'time': row[1],
                        'project': row[2],
                        'language': row[3],
                        'lines': row[4],
                        'branch': row[5],
                        'is_write': True if row[6] is 1 else False,
                        'plugin': row[7],
                    }
                loop = False
            except sqlite3.Error, e:
                log.debug(str(e))
                sleep(wait)
                tries -= 1
        conn.close()
        return action
